@@ -26,14 +26,23 @@
 #pragma once
 #endif // _MSC_VER > 1000
 class JNIMutex;
-class JNICriticalSection  
+/**
+ * This class holds a mutex which will be locked during the constructor and
+ * released during the destructor. If the object is created on the stack, this 
+ * garanties that the mutex will be released all the time if the block is left.
+ * Only one thread can enter all the critrical sections secured by the same 
+ * mutex.
+ */
+class JNICriticalSection
 {
 public:
-	JNICriticalSection(JNIMutex &mutex);
-	~JNICriticalSection();
-
+    JNICriticalSection(JNIMutex &mutex);
+    ~JNICriticalSection();
 private:
-	JNIMutex * f_mutex;
+    /**
+     * the mutex to be locked and released
+     */
+    JNIMutex * m_mutex;
 };
-
-#endif // !defined(AFX_JNICRITICALSECTION_H__601F33CC_3378_4B09_9917_6242857EF2B7__INCLUDED_)
+// !defined(AFX_JNICRITICALSECTION_H__601F33CC_3378_4B09_9917_6242857EF2B7__INCLUDED_)
+#endif 
