@@ -24,10 +24,12 @@ import org.tigris.subversion.javahl.*;
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.PrintWriter;
+import java.io.ByteArrayOutputStream;
 import java.util.Arrays;
+
 /**
- * this class tests the basic functionality of javahl binding. It was inspired
- * by the tests in subversion/tests/clients/cmdline/basic_tests.py
+ * Tests the basic functionality of javahl binding (inspired by the
+ * tests in subversion/tests/cmdline/basic_tests.py).
  */
 public class BasicTests extends SVNTests
 {
@@ -72,7 +74,49 @@ public class BasicTests extends SVNTests
     }
 
     /**
-     * test the basic SVNCLient.checkout functionality
+     * Test SVNClient.getVersion().
+     * @throws Throwable
+     */
+    public void testVersion() throws Throwable
+    {
+        try
+        {
+            Version version = client.getVersion();
+            String versionString = version.toString();
+            if (versionString == null || versionString.trim().length() == 0)
+            {
+                throw new Exception("Version string empty");
+            }
+        }
+        catch (Exception e)
+        {
+            fail("Version should always be available unless the " +
+                 "native libraries failed to initialize: " + e);
+        }
+    }
+
+    /**
+     * Tests Subversion path validation.
+     */
+    public void testPathValidation() throws Throwable
+    {
+        // Rather than segfaulting, JavaHL considers null an invalid path.
+        assertFalse("Path validation produced false-positive for null path",
+                    Path.isValid(null));
+
+        String path = "valid-path";
+        assertTrue("Validation check of valid path '" + path +
+                   "' should succeed", Path.isValid(path));
+
+        // File names cannot contain control characters.
+        path = "invalid-\u0001-path";
+        assertFalse("Validation check of invalid path '" + path +
+                    "' (which contains control characters) should fail",
+                    Path.isValid(path));
+    }
+
+    /**
+     * test the basic SVNClient.checkout functionality
      * @throws Throwable
      */
     public void testBasicCheckout() throws Throwable
@@ -86,7 +130,7 @@ public class BasicTests extends SVNTests
                     null, true);
             fail("missing exception");
         }
-        catch (ClientException e)
+        catch (ClientException expected)
         {
         }
         // modify file A/mu
@@ -529,21 +573,24 @@ public class BasicTests extends SVNTests
         OneTest thisTest = new OneTest();
 
         // create a lock file in A/B
-        File adminLock = new File(thisTest.getWorkingCopy(),"A/B/.svn/lock");
+        File adminLock = new File(thisTest.getWorkingCopy(),"A/B/" +
+                                  getAdminDirectoryName() + "/lock");
         PrintWriter pw = new PrintWriter(new FileOutputStream(adminLock));
         pw.print("stop looking!");
         pw.close();
         thisTest.getWc().setItemIsLocked("A/B", true);
 
         // create a lock file in A/D/G
-        adminLock = new File(thisTest.getWorkingCopy(),"A/D/G/.svn/lock");
+        adminLock = new File(thisTest.getWorkingCopy(),"A/D/G/" +
+                             getAdminDirectoryName() + "/lock");
         pw = new PrintWriter(new FileOutputStream(adminLock));
         pw.print("stop looking!");
         pw.close();
         thisTest.getWc().setItemIsLocked("A/D/G", true);
 
         // create a lock file in A/C
-        adminLock = new File(thisTest.getWorkingCopy(),"A/C/.svn/lock");
+        adminLock = new File(thisTest.getWorkingCopy(),"A/C/" +
+                             getAdminDirectoryName() + "/lock");
         pw = new PrintWriter(new FileOutputStream(adminLock));
         pw.print("stop looking!");
         pw.close();
@@ -778,7 +825,7 @@ public class BasicTests extends SVNTests
                     null, false);
             fail("missing exception");
         }
-        catch(ClientException e)
+        catch(ClientException expected)
         {
         }
 
@@ -790,7 +837,7 @@ public class BasicTests extends SVNTests
                     false);
             fail("missing exception");
         }
-        catch(ClientException e)
+        catch(ClientException expected)
         {
         }
 
@@ -802,7 +849,7 @@ public class BasicTests extends SVNTests
                     null, false);
             fail("missing exception");
         }
-        catch(ClientException e)
+        catch(ClientException expected)
         {
         }
 
@@ -814,7 +861,7 @@ public class BasicTests extends SVNTests
                     false);
             fail("missing exception");
         }
-        catch(ClientException e)
+        catch(ClientException expected)
         {
         }
 
@@ -826,7 +873,7 @@ public class BasicTests extends SVNTests
                     false);
             fail("missing exception");
         }
-        catch(ClientException e)
+        catch(ClientException expected)
         {
         }
 
@@ -838,7 +885,7 @@ public class BasicTests extends SVNTests
                     false);
             fail("missing exception");
         }
-        catch(ClientException e)
+        catch(ClientException expected)
         {
         }
 
@@ -850,7 +897,7 @@ public class BasicTests extends SVNTests
                     null, false);
             fail("missing exception");
         }
-        catch(ClientException e)
+        catch(ClientException expected)
         {
         }
 
@@ -862,7 +909,7 @@ public class BasicTests extends SVNTests
                     false);
             fail("missing exception");
         }
-        catch(ClientException e)
+        catch(ClientException expected)
         {
         }
 
@@ -873,7 +920,7 @@ public class BasicTests extends SVNTests
                     false);
             fail("missing exception");
         }
-        catch(ClientException e)
+        catch(ClientException expected)
         {
         }
 
@@ -950,7 +997,16 @@ public class BasicTests extends SVNTests
         pw.close();
         client.remove(new String[] {file.getAbsolutePath()}, null, true);
         assertFalse("failed to remove unversioned file foo", file.exists());
-        client.remove(new String[] {file.getAbsolutePath()}, null, true);
+
+        try
+        {
+            // delete non-existant file foo
+            client.remove(new String[] {file.getAbsolutePath()}, null, true);
+            fail("missing exception");
+        }
+        catch(ClientException expected)
+        {
+        }
 
         // delete file iota in the repository
         addExpectedCommitItem(null, thisTest.getUrl(), "iota", NodeKind.none,
@@ -1143,6 +1199,32 @@ public class BasicTests extends SVNTests
         assertTrue("content changed", Arrays.equals(content, testContent));
     }
 
+    /**
+     * test the basic SVNClient.fileContent functionality
+     * @throws Throwable
+     */
+    public void testBasicCatStream() throws Throwable
+    {
+        // create the working copy
+        OneTest thisTest = new OneTest();
+
+        // modify A/mu
+        File mu = new File(thisTest.getWorkingCopy(), "A/mu");
+        PrintWriter pw = new PrintWriter(new FileOutputStream(mu, true));
+        pw.print("some text");
+        pw.close();
+        // get the content from the repository
+        ByteArrayOutputStream baos = new ByteArrayOutputStream();
+        client.streamFileContent(thisTest.getWCPath() + "/A/mu", null, null,
+                                 100, baos);
+        
+        byte[] content = baos.toByteArray();
+        byte[] testContent = thisTest.getWc().getItemContent("A/mu").getBytes();
+
+        // the content should be the same
+        assertTrue("content changed", Arrays.equals(content, testContent));
+    }
+
     /**
      * test the basic SVNClient.list functionality
      * @throws Throwable
@@ -1339,6 +1421,17 @@ public class BasicTests extends SVNTests
         assertEquals("rev number from commit",-1, client.commit(
                 new String[]{thisTest.getWCPath()},"message", true));
         assertEquals("file should be read write now", true, f.canWrite());
+
+        try
+        {
+            // Attempt to lock an invalid path
+            client.lock(new String[]{thisTest.getWCPath()+"/A/mu2"}, "comment",
+                        false);
+            fail("missing exception");
+        }
+        catch (ClientException expected)
+        {
+        }
     }
 
     /**
@@ -1359,4 +1452,127 @@ public class BasicTests extends SVNTests
                              new Revision.Number(1), true);
         assertEquals("this should return 21 info objects", 21, infos.length);
     }
+
+    /**
+     * test the basic SVNClient.merge functionality
+     * @throws Throwable
+     * @since 1.2
+     */
+    public void testBasicMerge() throws Throwable
+    {
+        // build the test setup
+        OneTest thisTest = new OneTest();
+        
+        // create branches directory in the repository
+        addExpectedCommitItem(null, thisTest.getUrl(), "branches", NodeKind.none,
+              CommitItemStateFlags.Add);
+        client.mkdir(new String[]{thisTest.getUrl() + "/branches"}, "log_msg");
+
+        // copy A to branches
+        addExpectedCommitItem(null, thisTest.getUrl(), "branches/A", NodeKind.none,
+                CommitItemStateFlags.Add);
+        client.copy(thisTest.getUrl() + "/A", thisTest.getUrl() + "/branches/A", "create A branch", Revision.HEAD);
+
+        // update the WC so that it has the branches folder
+        client.update(thisTest.getWCPath(), Revision.HEAD, true);
+
+        // modify file A/mu
+        File mu = new File(thisTest.getWorkingCopy(), "A/mu");
+        PrintWriter muPW = new PrintWriter(new FileOutputStream(mu, true));
+        muPW.print("appended mu text");
+        muPW.close();
+        thisTest.getWc().setItemWorkingCopyRevision("A/mu", 4);
+        thisTest.getWc().setItemContent("A/mu",
+                thisTest.getWc().getItemContent("A/mu") + "appended mu text");
+        addExpectedCommitItem(thisTest.getWCPath(),
+                thisTest.getUrl(), "A/mu",NodeKind.file,
+                CommitItemStateFlags.TextMods);
+
+        // modify file A/D/G/rho
+        File rho = new File(thisTest.getWorkingCopy(), "A/D/G/rho");
+        PrintWriter rhoPW = new PrintWriter(new FileOutputStream(rho, true));
+        rhoPW.print("new appended text for rho");
+        rhoPW.close();
+        thisTest.getWc().setItemWorkingCopyRevision("A/D/G/rho", 4);
+        thisTest.getWc().setItemContent("A/D/G/rho",
+                thisTest.getWc().getItemContent("A/D/G/rho")
+                + "new appended text for rho");
+        addExpectedCommitItem(thisTest.getWCPath(),
+                thisTest.getUrl(), "A/D/G/rho",NodeKind.file,
+                CommitItemStateFlags.TextMods);
+        // commit the changes
+        assertEquals("wrong revision number from commit",
+              client.commit(new String[]{thisTest.getWCPath()}, "log msg",
+                      true), 4);
+
+        // merge changes in A to branches/A
+        String branchPath = thisTest.getWCPath() + "/branches/A";
+        String modUrl = thisTest.getUrl() + "/A";
+        // test --dry-run
+        client.merge(modUrl, new Revision.Number(2), modUrl, Revision.HEAD, branchPath, false, true, false, true);
+
+        // now do the real merge
+        client.merge(modUrl, new Revision.Number(2), modUrl, Revision.HEAD, branchPath, false, true, false, false);
+        
+        // commit the changes so that we can verify merge
+        addExpectedCommitItem(thisTest.getWCPath(),
+                thisTest.getUrl(), "branches/A/mu",NodeKind.file,
+                CommitItemStateFlags.TextMods);
+        addExpectedCommitItem(thisTest.getWCPath(),
+                thisTest.getUrl(), "branches/A/D/G/rho",NodeKind.file,
+                CommitItemStateFlags.TextMods);
+        assertEquals("wrong revision number from commit",
+              client.commit(new String[]{thisTest.getWCPath()}, "log msg",
+                      true), 5);
+ 
+    }
+    /**
+     * test the basic SVNClient.isAdminDirectory functionality
+     * @throws Throwable
+     * @since 1.2
+     */
+    public void testBasicIsAdminDirectory() throws Throwable
+    {
+        // build the test setup
+        OneTest thisTest = new OneTest();
+        Notify2 notify = new Notify2(){
+            public void onNotify(NotifyInformation info)
+            {
+                client.isAdminDirectory(".svn");
+            }
+        };
+        client.notification2(notify);
+        // update the test
+        assertEquals("wrong revision number from update",
+                client.update(thisTest.getWCPath(), null, true), 1);
+    }
+    public void testBasicCancelOperation() throws Throwable
+    {
+        // build the test setup
+        OneTest thisTest = new OneTest();
+        Notify2 notify = new Notify2(){
+            public void onNotify(NotifyInformation info)
+            {
+                try
+                {
+                    client.cancelOperation();
+                }
+                catch (ClientException e)
+                {
+                    fail(e.getMessage());
+                }
+            }
+        };
+        client.notification2(notify);
+        // update the test to try to cancel an operation
+        try
+        {
+            client.update(thisTest.getWCPath(), null, true);
+            fail("missing exception for canceled operation");
+        }
+        catch (ClientException e)
+        {
+            // this is expected
+        }
+    }
 }
