@@ -1,7 +1,7 @@
 /* fs.c --- creating, opening and closing filesystems
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -18,7 +18,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <errno.h>              /* for EINVAL */
 
 #include <apr_general.h>
 #include <apr_pools.h>
@@ -40,21 +39,19 @@
 #include "../libsvn_fs/fs-loader.h"
 
 /* A prefix for the pool userdata variables used to hold
-   per-repository mutexes.  See fs_serialized_init. */
-#if APR_HAS_THREADS
-#define SVN_FSFS_LOCK_USERDATA_PREFIX "svn-fsfs-lock-"
-#endif
+   per-filesystem shared data.  See fs_serialized_init. */
+#define SVN_FSFS_SHARED_USERDATA_PREFIX "svn-fsfs-shared-"
 
 
 
 /* If filesystem FS is already open, then return an
    SVN_ERR_FS_ALREADY_OPEN error.  Otherwise, return zero.  */
 static svn_error_t *
-check_already_open (svn_fs_t *fs)
+check_already_open(svn_fs_t *fs)
 {
   if (fs->fsap_data)
-    return svn_error_create (SVN_ERR_FS_ALREADY_OPEN, 0,
-                             _("Filesystem object already open"));
+    return svn_error_create(SVN_ERR_FS_ALREADY_OPEN, 0,
+                            _("Filesystem object already open"));
   else
     return SVN_NO_ERROR;
 }
@@ -62,55 +59,68 @@ check_already_open (svn_fs_t *fs)
 
 
 static svn_error_t *
-fs_serialized_init (svn_fs_t *fs, apr_pool_t *common_pool, apr_pool_t *pool)
+fs_serialized_init(svn_fs_t *fs, apr_pool_t *common_pool, apr_pool_t *pool)
 {
-#if APR_HAS_THREADS
   fs_fs_data_t *ffd = fs->fsap_data;
   const char *key;
   void *val;
-  apr_thread_mutex_t *lock;
+  fs_fs_shared_data_t *ffsd;
   apr_status_t status;
 
-  /* POSIX fcntl locks are per-process, so we need a mutex for
-     intra-process synchronization when grabbing the repository write
-     lock.  Fetch a repository-specific lock from the pool user data.
-
-     Note that we are allocating a small amount of long-lived data for
+  /* Note that we are allocating a small amount of long-lived data for
      each separate repository opened during the lifetime of the
      svn_fs_initialize pool.  It's unlikely that anyone will notice
-     this modest expenditure; the alternative is to put each lock into
-     a reference-counted structure allocated in a subpool, and add a
-     serialized deconstructor to the FS vtable.  That's more machinery
-     than it's worth.
+     the modest expenditure; the alternative is to allocate each structure
+     in a subpool, add a reference-count, and add a serialized deconstructor
+     to the FS vtable.  That's more machinery than it's worth.
 
      Using the uuid to obtain the lock creates a corner case if a
      caller uses svn_fs_set_uuid on the repository in a process where
      other threads might be using the same repository through another
      FS object.  The only real-world consumer of svn_fs_set_uuid is
      "svnadmin load", so this is a low-priority problem, and we don't
-     know of a better way of associating a mutex with the
+     know of a better way of associating such data with the
      repository. */
 
-  key = apr_pstrcat (pool, SVN_FSFS_LOCK_USERDATA_PREFIX, ffd->uuid,
-                     (char *) NULL);
-  status = apr_pool_userdata_get (&val, key, common_pool);
+  key = apr_pstrcat(pool, SVN_FSFS_SHARED_USERDATA_PREFIX, ffd->uuid,
+                    (char *) NULL);
+  status = apr_pool_userdata_get(&val, key, common_pool);
   if (status)
-    return svn_error_wrap_apr (status, _("Can't fetch FSFS mutex"));
-  lock = val;
-  if (!lock)
+    return svn_error_wrap_apr(status, _("Can't fetch FSFS shared data"));
+  ffsd = val;
+
+  if (!ffsd)
     {
-      status = apr_thread_mutex_create (&lock, APR_THREAD_MUTEX_DEFAULT,
-                                        common_pool);
+      ffsd = apr_pcalloc(common_pool, sizeof(*ffsd));
+      ffsd->common_pool = common_pool;
+
+#if APR_HAS_THREADS
+      /* POSIX fcntl locks are per-process, so we need a mutex for
+         intra-process synchronization when grabbing the repository write
+         lock. */
+      status = apr_thread_mutex_create(&ffsd->fs_write_lock,
+                                       APR_THREAD_MUTEX_DEFAULT, common_pool);
       if (status)
-        return svn_error_wrap_apr (status, _("Can't create FSFS mutex"));
-      key = apr_pstrdup (common_pool, key);
-      status = apr_pool_userdata_set (lock, key, NULL, common_pool);
+        return svn_error_wrap_apr(status,
+                                  _("Can't create FSFS write-lock mutex"));
+
+      /* We also need a mutex for synchronising access to the active
+         transaction list and free transaction pointer. */
+      status = apr_thread_mutex_create(&ffsd->txn_list_lock,
+                                       APR_THREAD_MUTEX_DEFAULT, common_pool);
       if (status)
-        return svn_error_wrap_apr (status, _("Can't store FSFS mutex"));
-    }
-  ffd->lock = lock;
+        return svn_error_wrap_apr(status,
+                                  _("Can't create FSFS txn list mutex"));
 #endif
 
+      key = apr_pstrdup(common_pool, key);
+      status = apr_pool_userdata_set(ffsd, key, NULL, common_pool);
+      if (status)
+        return svn_error_wrap_apr(status, _("Can't store FSFS shared data"));
+    }
+
+  ffd->shared = ffsd;
+
   return SVN_NO_ERROR;
 }
 
@@ -120,8 +130,8 @@ fs_serialized_init (svn_fs_t *fs, apr_pool_t *common_pool, apr_pool_t *pool)
    has no effect for fsfs backed Subversion filesystems.  It conforms
    to the fs_library_vtable_t.bdb_set_errcall() API. */
 static svn_error_t *
-fs_set_errcall (svn_fs_t *fs,
-                void (*db_errcall_fcn) (const char *errpfx, char *msg))
+fs_set_errcall(svn_fs_t *fs,
+               void (*db_errcall_fcn)(const char *errpfx, char *msg))
 {
 
   return SVN_NO_ERROR;
@@ -159,17 +169,17 @@ static fs_vtable_t fs_vtable = {
    fsfs-backed Subversion filesystem at path PATH and link it into
    *FS.  Perform temporary allocations in POOL. */
 static svn_error_t *
-fs_create (svn_fs_t *fs, const char *path, apr_pool_t *pool)
+fs_create(svn_fs_t *fs, const char *path, apr_pool_t *pool)
 {
   fs_fs_data_t *ffd;
 
-  SVN_ERR (check_already_open (fs));
+  SVN_ERR(check_already_open(fs));
 
-  ffd = apr_pcalloc (fs->pool, sizeof (*ffd));
+  ffd = apr_pcalloc(fs->pool, sizeof(*ffd));
   fs->vtable = &fs_vtable;
   fs->fsap_data = ffd;
 
-  SVN_ERR (svn_fs_fs__create (fs, path, pool));
+  SVN_ERR(svn_fs_fs__create(fs, path, pool));
 
   return SVN_NO_ERROR;
 }
@@ -183,15 +193,15 @@ fs_create (svn_fs_t *fs, const char *path, apr_pool_t *pool)
    correct vtable for the filesystem.  Use POOL for any temporary
    allocations. */
 static svn_error_t *
-fs_open (svn_fs_t *fs, const char *path, apr_pool_t *pool)
+fs_open(svn_fs_t *fs, const char *path, apr_pool_t *pool)
 {
   fs_fs_data_t *ffd;
 
-  ffd = apr_pcalloc (fs->pool, sizeof (*ffd));
+  ffd = apr_pcalloc(fs->pool, sizeof(*ffd));
   fs->vtable = &fs_vtable;
   fs->fsap_data = ffd;
 
-  SVN_ERR (svn_fs_fs__open (fs, path, pool));
+  SVN_ERR(svn_fs_fs__open(fs, path, pool));
 
   return SVN_NO_ERROR;
 }
@@ -203,12 +213,12 @@ fs_open (svn_fs_t *fs, const char *path, apr_pool_t *pool)
    The CLEAN_LOGS argument is ignored and included for Subversion
    1.0.x compatibility.  Perform all temporary allocations in POOL. */
 static svn_error_t *
-fs_hotcopy (const char *src_path, 
-            const char *dest_path, 
-            svn_boolean_t clean_logs, 
-            apr_pool_t *pool)
+fs_hotcopy(const char *src_path, 
+           const char *dest_path, 
+           svn_boolean_t clean_logs, 
+           apr_pool_t *pool)
 {
-  SVN_ERR (svn_fs_fs__hotcopy (src_path, dest_path, pool));
+  SVN_ERR(svn_fs_fs__hotcopy(src_path, dest_path, pool));
 
   return SVN_NO_ERROR;
 }
@@ -219,8 +229,8 @@ fs_hotcopy (const char *src_path,
    no effect for fsfs backed Subversion filesystems.  It conforms to
    the fs_library_vtable_t.bdb_recover() API. */
 static svn_error_t *
-fs_recover (const char *path,
-            apr_pool_t *pool)
+fs_recover(const char *path,
+           apr_pool_t *pool)
 {
   /* This is a no-op for FSFS. */
 
@@ -234,13 +244,13 @@ fs_recover (const char *path,
    has no effect for fsfs backed Subversion filesystems.  It conforms
    to the fs_library_vtable_t.bdb_logfiles() API. */
 static svn_error_t *
-fs_logfiles (apr_array_header_t **logfiles,
-             const char *path,
-             svn_boolean_t only_unused,
-             apr_pool_t *pool)
+fs_logfiles(apr_array_header_t **logfiles,
+            const char *path,
+            svn_boolean_t only_unused,
+            apr_pool_t *pool)
 {
   /* A no-op for FSFS. */
-  *logfiles = apr_array_make (pool, 0, sizeof (const char *));
+  *logfiles = apr_array_make(pool, 0, sizeof(const char *));
 
   return SVN_NO_ERROR;
 }
@@ -252,11 +262,11 @@ fs_logfiles (apr_array_header_t **logfiles,
 /* Delete the filesystem located at path PATH.  Perform any temporary
    allocations in POOL. */
 static svn_error_t *
-fs_delete_fs (const char *path,
-              apr_pool_t *pool)
+fs_delete_fs(const char *path,
+             apr_pool_t *pool)
 {
   /* Remove everything. */
-  SVN_ERR (svn_io_remove_dir (path, pool));
+  SVN_ERR(svn_io_remove_dir(path, pool));
 
   return SVN_NO_ERROR;
 }
@@ -266,7 +276,7 @@ fs_delete_fs (const char *path,
 /* Miscellany */
 
 const char *
-svn_fs_fs__canonicalize_abspath (const char *path, apr_pool_t *pool)
+svn_fs_fs__canonicalize_abspath(const char *path, apr_pool_t *pool)
 {
   char *newpath;
   int path_len;
@@ -279,12 +289,12 @@ svn_fs_fs__canonicalize_abspath (const char *path, apr_pool_t *pool)
   
   /* Empty PATH?  That's just "/". */
   if (! *path)
-    return apr_pstrdup (pool, "/");
+    return apr_pstrdup(pool, "/");
 
   /* Now, the fun begins.  Alloc enough room to hold PATH with an
      added leading '/'. */
-  path_len = strlen (path);
-  newpath = apr_pcalloc (pool, path_len + 2);
+  path_len = strlen(path);
+  newpath = apr_pcalloc(pool, path_len + 2);
 
   /* No leading slash?  Fix that. */
   if (*path != '/')
@@ -324,13 +334,13 @@ svn_fs_fs__canonicalize_abspath (const char *path, apr_pool_t *pool)
 }
 
 static const svn_version_t *
-fs_version (void)
+fs_version(void)
 {
   SVN_VERSION_BODY;
 }
 
 static const char *
-fs_get_description (void)
+fs_get_description(void)
 {
   return _("Module for working with a plain file (FSFS) repository.");
 }
@@ -351,8 +361,8 @@ static fs_library_vtable_t library_vtable = {
 };
 
 svn_error_t *
-svn_fs_fs__init (const svn_version_t *loader_version,
-                 fs_library_vtable_t **vtable)
+svn_fs_fs__init(const svn_version_t *loader_version,
+                fs_library_vtable_t **vtable)
 {
   static const svn_version_checklist_t checklist[] =
     {
@@ -364,10 +374,10 @@ svn_fs_fs__init (const svn_version_t *loader_version,
   /* Simplified version check to make sure we can safely use the
      VTABLE parameter. The FS loader does a more exhaustive check. */
   if (loader_version->major != SVN_VER_MAJOR)
-    return svn_error_createf (SVN_ERR_VERSION_MISMATCH, NULL,
-                              _("Unsupported FS loader version (%d) for fsfs"),
-                              loader_version->major);
-  SVN_ERR (svn_ver_check_list (fs_version(), checklist));
+    return svn_error_createf(SVN_ERR_VERSION_MISMATCH, NULL,
+                             _("Unsupported FS loader version (%d) for fsfs"),
+                             loader_version->major);
+  SVN_ERR(svn_ver_check_list(fs_version(), checklist));
 
   *vtable = &library_vtable;
   return SVN_NO_ERROR;
