@@ -34,14 +34,82 @@ extern "C" {
 /* The format number of this filesystem.
    This is independent of the repository format number, and
    independent of any other FS back ends. */
-#define SVN_FS_FS__FORMAT_NUMBER   1
+#define SVN_FS_FS__FORMAT_NUMBER   2
 
+/* The minimum format number that supports svndiff version 1.  */
+#define SVN_FS_FS__MIN_SVNDIFF1_FORMAT 2
+
+/* Maximum number of directories to cache dirents for. 
+   This *must* be a power of 2 for DIR_CACHE_ENTRIES_INDEX
+   to work.  */
+#define NUM_DIR_CACHE_ENTRIES 128
+#define DIR_CACHE_ENTRIES_MASK(x) ((x) & (NUM_DIR_CACHE_ENTRIES - 1))
+
+
+/* Private FSFS-specific data shared between all svn_txn_t objects that
+   relate to a particular transaction in a filesystem (as identified
+   by transaction id and filesystem UUID).  Objects of this type are
+   allocated in their own subpool of the common pool. */
+struct fs_fs_shared_txn_data_t;
+typedef struct fs_fs_shared_txn_data_t
+{
+  /* The next transaction in the list, or NULL if there is no following
+     transaction. */
+  struct fs_fs_shared_txn_data_t *next;
+
+  /* This transaction's ID.  This is in the form <rev>-<uniqueifier>,
+     where <uniqueifier> runs from 0-99999 (see create_txn_dir() in
+     fs_fs.c). */
+  char txn_id[18];
+
+  /* Whether the transaction's prototype revision file is locked for
+     writing by any thread in this process (including the current
+     thread; recursive locks are not permitted).  This is effectively
+     a non-recursive mutex. */
+  svn_boolean_t being_written;
+
+  /* The pool in which this object has been allocated; a subpool of the
+     common pool. */
+  apr_pool_t *pool;
+} fs_fs_shared_txn_data_t;
+
+
+/* Private FSFS-specific data shared between all svn_fs_t objects that
+   relate to a particular filesystem, as identified by filesystem UUID.
+   Objects of this type are allocated in the common pool. */
+typedef struct
+{
+  /* A list of shared transaction objects for each transaction that is
+     currently active, or NULL if none are.  All access to this list,
+     including the contents of the objects stored in it, is synchronised
+     under TXN_LIST_LOCK. */
+  fs_fs_shared_txn_data_t *txns;
+
+  /* A free transaction object, or NULL if there is no free object.
+     Access to this object is synchronised under TXN_LIST_LOCK. */
+  fs_fs_shared_txn_data_t *free_txn;
+
+#if APR_HAS_THREADS
+  /* A lock for intra-process synchronization when accessing the TXNS list. */
+  apr_thread_mutex_t *txn_list_lock;
+
+  /* A lock for intra-process synchronization when grabbing the
+     repository write lock. */
+  apr_thread_mutex_t *fs_write_lock;
+#endif
+
+  /* The common pool, under which this object is allocated, subpools
+     of which are used to allocate the transaction objects. */
+  apr_pool_t *common_pool;
+} fs_fs_shared_data_t;
+
+/* Private (non-shared) FSFS-specific data for each svn_fs_t object. */
 typedef struct
 {
   /* A cache of the last directory opened within the filesystem. */
-  svn_fs_id_t *dir_cache_id;
-  apr_hash_t *dir_cache;
-  apr_pool_t *dir_cache_pool;
+  svn_fs_id_t *dir_cache_id[NUM_DIR_CACHE_ENTRIES];
+  apr_hash_t *dir_cache[NUM_DIR_CACHE_ENTRIES];
+  apr_pool_t *dir_cache_pool[NUM_DIR_CACHE_ENTRIES];
 
   /* The format number of this FS. */
   int format;
@@ -49,12 +117,8 @@ typedef struct
   /* The uuid of this FS. */
   const char *uuid;
 
-#if APR_HAS_THREADS
-  /* A lock for intra-process synchronization when grabbing the
-     repository write lock.  Common to all repositories with the same
-     uuid; discovered using the serialized_init function. */
-  apr_thread_mutex_t *lock;
-#endif
+  /* Data shared between all svn_fs_t objects for a given filesystem. */
+  fs_fs_shared_data_t *shared;
 } fs_fs_data_t;
 
 
@@ -67,17 +131,7 @@ typedef struct
    filesystem paths should begin with '/', and all redundant and trailing '/'
    characters be removed.  */
 const char *
-svn_fs_fs__canonicalize_abspath (const char *path, apr_pool_t *pool);
-
-
-/*** Filesystem Revision ***/
-typedef struct
-{
-  /* id of the transaction that was committed to create this
-     revision. */
-  const char *txn_id;
-
-} revision_t;
+svn_fs_fs__canonicalize_abspath(const char *path, apr_pool_t *pool);
 
 
 /*** Transaction Kind ***/
@@ -113,14 +167,6 @@ typedef struct
 
 } transaction_t;
 
-/*** Copy Kind ***/
-typedef enum
-{
-  copy_kind_real = 1, /* real copy */
-  copy_kind_soft      /* soft copy */
-
-} copy_kind_t;
-
 
 /*** Representation ***/
 typedef struct
@@ -193,60 +239,6 @@ typedef struct
 } node_revision_t;
 
 
-/*** Representation Kind ***/
-typedef enum
-{
-  rep_kind_fulltext = 1, /* fulltext */
-  rep_kind_delta         /* delta */
-
-} rep_kind_t;
-
-
-/*** "Delta" Offset/Window Chunk ***/
-typedef struct 
-{
-  /* diff format version number ### at this point, "svndiff" is the
-     only format used. */
-  apr_byte_t version;
-
-  /* starting offset of the data represented by this chunk */
-  svn_filesize_t offset;
-
-  /* string-key to which this representation points. */
-  const char *string_key; 
-
-  /* size of the fulltext data represented by this delta window. */
-  apr_size_t size;
-
-  /* represenatation-key to use when needed source data for
-     undeltification. */
-  const char *rep_key;
-
-  /* apr_off_t rep_offset;  ### not implemented */
-
-} rep_delta_chunk_t;
-
-
-
-
-/*** Copy ***/
-typedef struct
-{
-  /* What kind of copy occurred. */
-  copy_kind_t kind;
-
-  /* Path of copy source. */
-  const char *src_path;
-
-  /* Transaction id of copy source. */
-  const char *src_txn_id;
-
-  /* Node-revision of copy destination. */
-  const svn_fs_id_t *dst_noderev_id;
-
-} copy_t;
-
-
 /*** Change ***/
 typedef struct
 {
