@@ -67,7 +67,8 @@ enum {
 enum {
   SVN_PROPID_baseline_relative_path = 1,
   SVN_PROPID_md5_checksum,
-  SVN_PROPID_repository_uuid
+  SVN_PROPID_repository_uuid,
+  SVN_PROPID_deadprop_count
 };
 
 static const dav_liveprop_spec dav_svn_props[] =
@@ -96,6 +97,7 @@ static const dav_liveprop_spec dav_svn_props[] =
   SVN_RO_SVN_PROP(baseline_relative_path, baseline-relative-path),
   SVN_RO_SVN_PROP(md5_checksum, md5-checksum),
   SVN_RO_SVN_PROP(repository_uuid, repository-uuid),
+  SVN_RO_SVN_PROP(deadprop_count, deadprop-count),
 
   { 0 } /* sentinel */
 };
@@ -233,8 +235,8 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
             format = dav_svn_time_format_rfc1123;
           }
 
-        if (0 != dav_svn_get_last_modified_time (&datestring, &timeval,
-                                                 resource, format, p))
+        if (0 != dav_svn_get_last_modified_time(&datestring, &timeval,
+                                                resource, format, p))
           {
             return DAV_PROP_INSERT_NOTDEF;
           }
@@ -354,9 +356,9 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           }
         else
           {
-            if ((serr = svn_fs_node_prop (&pval, resource->info->root.root,
-                                          resource->info->repos_path,
-                                          SVN_PROP_MIME_TYPE, p)))
+            if ((serr = svn_fs_node_prop(&pval, resource->info->root.root,
+                                         resource->info->repos_path,
+                                         SVN_PROP_MIME_TYPE, p)))
               {
                 svn_error_clear(serr);
                 pval = NULL;
@@ -370,7 +372,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
             else
               mime_type = "text/plain"; /* default for file */
 
-            if ((serr = svn_mime_type_validate (mime_type, p)))
+            if ((serr = svn_mime_type_validate(mime_type, p)))
               {
                 /* Probably serr->apr == SVN_ERR_BAD_MIME_TYPE, but
                    there's no point even checking.  No matter what the
@@ -542,7 +544,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
               break;
             }
 
-          value = svn_md5_digest_to_cstring (digest, p);
+          value = svn_md5_digest_to_cstring(digest, p);
 
           if (! value)
             return DAV_PROP_INSERT_NOTSUPP;
@@ -563,6 +565,30 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
         }
       break;
 
+    case SVN_PROPID_deadprop_count:
+      {
+        unsigned int propcount;
+        apr_hash_t *proplist;  
+      
+        if (resource->type != DAV_RESOURCE_TYPE_REGULAR)
+          return DAV_PROP_INSERT_NOTSUPP;
+
+        serr = svn_fs_node_proplist(&proplist,
+                                    resource->info->root.root,
+                                    resource->info->repos_path, p);
+        if (serr != NULL)
+          {
+            /* ### what to do? */
+            svn_error_clear(serr);
+            value = "###error###";
+            break;  
+          }
+         
+        propcount = apr_hash_count(proplist);
+        value = apr_psprintf(p, "%u", propcount);
+        break;
+      }
+      
     default:
       /* ### what the heck was this property? */
       return DAV_PROP_INSERT_NOTDEF;
@@ -717,11 +743,11 @@ void dav_svn_register_uris(apr_pool_t *p)
 }
 
 
-int dav_svn_get_last_modified_time (const char **datestring,
-                                    apr_time_t *timeval,
-                                    const dav_resource *resource,
-                                    enum dav_svn_time_format format,
-                                    apr_pool_t *pool)
+int dav_svn_get_last_modified_time(const char **datestring,
+                                   apr_time_t *timeval,
+                                   const dav_resource *resource,
+                                   enum dav_svn_time_format format,
+                                   apr_pool_t *pool)
 {
   svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
   svn_string_t *committed_date = NULL;
