@@ -39,6 +39,25 @@ def write_restrictive_svnserve_conf(repo_dir):
            "password-db = passwd\nauthz-db = authz\n")
   fp.close()
 
+def write_authz_file(sbox, rules, sections=None):
+  """Write an authz file to SBOX, appropriate for the RA method used,
+with authorizations rules RULES mapping paths to strings containing
+the rules. You can add sections SECTIONS (ex. groups, aliases...) with 
+an appropriate list of mappings.
+"""
+  fp = open(sbox.authz_file, 'w')
+  if sbox.repo_url.startswith("http"):
+    prefix = sbox.name + ":"
+  else:
+    prefix = ""
+  if sections:
+    for p, r in sections.items():
+      fp.write("[%s]\n%s\n" % (p, r))  
+
+  for p, r in rules.items():
+    fp.write("[%s%s]\n%s\n" % (prefix, p, r))
+  fp.close()
+
 def skip_test_when_no_authz_available():
   "skip this test when authz is not available"
   if svntest.main.test_area_url.startswith('file://'):
@@ -134,8 +153,10 @@ def broken_authz_file(sbox):
   
   skip_test_when_no_authz_available()
   
+  # No characters but 'r', 'w', and whitespace are allowed as a value
+  # in an authz rule.
   fp = open(sbox.authz_file, 'w')
-  fp.write("[/]\njrandom = rw zot\n")
+  fp.write("[/]\njrandom = rw # End-line comments disallowed\n")
   fp.close()
   
   write_restrictive_svnserve_conf(svntest.main.current_repo_dir)
@@ -753,6 +774,73 @@ def authz_log_and_tracing_test(sbox):
   svntest.actions.run_and_verify_svn("", None, expected_err,
                                     'diff', '-r', '2:4', D_url+'/rho')  
   
+def authz_validate(sbox):
+  "test the authz validation rules"
+
+  skip_test_when_no_authz_available()
+
+  sbox.build(create_wc = False)
+
+  write_restrictive_svnserve_conf(sbox.repo_dir)
+
+  A_url = sbox.repo_url + '/A'
+
+  # If any of the validate rules fail, the authz isn't loaded so there's no 
+  # access at all to the repository.
+
+  # Test 1: Undefined group
+  write_authz_file(sbox, { "/"  : "* = r",
+                           "/A/B" : "@undefined_group = rw" })
+
+  if sbox.repo_url.startswith("http"):
+    expected_err = ".*403 Forbidden.*"
+  else:
+    expected_err = ".*@undefined_group.*"
+
+  # validation of this authz file should fail, so no repo access
+  svntest.actions.run_and_verify_svn("ls remote folder",
+                                     None, expected_err,
+                                     'ls',
+                                     '--username', svntest.main.wc_author,
+                                     '--password', svntest.main.wc_passwd,
+                                     A_url)
+
+  # Test 2: Circular dependency
+  write_authz_file(sbox, { "/"  : "* = r" },
+                         { "groups" : """admins = admin1, admin2, @devs
+devs1 = @admins, dev1
+devs2 = @admins, dev2
+devs = @devs1, dev3, dev4""" })
+
+  if sbox.repo_url.startswith("http"):
+    expected_err = ".*403 Forbidden.*"
+  else:
+    expected_err = ".*Circular dependency.*"
+
+  # validation of this authz file should fail, so no repo access
+  svntest.actions.run_and_verify_svn("ls remote folder",
+                                     None, expected_err,
+                                     'ls',
+                                     '--username', svntest.main.wc_author,
+                                     '--password', svntest.main.wc_passwd,
+                                     A_url)
+
+  # Test 3: Group including other group 2 times (issue 2684)
+  write_authz_file(sbox, { "/"  : "* = r" },
+                         { "groups" : """admins = admin1, admin2
+devs1 = @admins, dev1
+devs2 = @admins, dev2
+users = @devs1, @devs2, user1, user2""" })
+
+  # validation of this authz file should fail, so no repo access
+  svntest.actions.run_and_verify_svn("ls remote folder",
+                                      ['B/\n', 'C/\n', 'D/\n', 'mu\n'],
+                                      [],
+                                     'ls',
+                                     '--username', svntest.main.wc_author,
+                                     '--password', svntest.main.wc_passwd,
+                                     A_url)
+
 ########################################################################
 # Run the tests
 
@@ -770,6 +858,7 @@ test_list = [ None,
               authz_log_and_tracing_test,
               authz_checkout_and_update_test,
               authz_partial_export_test,
+              authz_validate,
              ]
 
 if __name__ == '__main__':
